
/******************************************************************
 *
 * Name:	Plugin_procs.c
 * 
 * Description: Audio plugin for OpenDVE with background sending 
 *              audio process  
 * 
 * Author:	Tomasz Stachowiak
 *
 * Date:	03/10/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>

#include <dve/dve.h>

#include <Xm/Xm.h>

#include <sys/time.h>
#include <dmedia/audio.h>

/*
 *    stuff for sproc(2) system call to fork a second thread.
 */
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/schedctl.h>

/*
 *    the following define enables 4.3BSD behavior for the kill(3B)
 *    system call, and the include file is necessary for this function.
 */
#define _BSD_SIGNALS
#include <signal.h>

#include "TSAudioPluginMotif.h"
#include "AudioPluginProtocol.h"
#include "gsm.h"
#include "adpcm.h"


/***********************************************************************
 *
 * GLOBAL VARIABLES
 *
 ***********************************************************************/

/* For audio library functions */
extern ALport audio_output_ports[];
extern ALconfig conf[];

/* For DVE */
extern DveSession dveSession;      

/* For Motif */
XtAppContext app;
XtWorkProcId workproc;

/* Mine */
int current_state;
extern code_type;


/* LOCAL VARIABLES */
/* For GSM codec */
gsm handle;

/* For ADPCM */
struct adpcm_state adpcmstates[MAX_USERS];
struct adpcm_state adpcmsendstate;

/*
 * GSM specific functions 
 */
gsm_init()
  {
     handle = gsm_create();
  }

gsm_close()
  {
     gsm_destroy(handle);
  }


initADPCMstates()
{
  int i;
  
  adpcmsendstate.valprev = 0;
  adpcmsendstate.index = 0;

  for(i=0; i<MAX_USERS; i++)
    {
      adpcmstates[i].valprev = 0;
      adpcmstates[i].index = 0;
    }
}

/* 
 * Output audio ports functions
 */
init_audio_output_ports()
  {
     int i;

     for(i=0; i<100; i++)
	audio_output_ports[i] = 0;
  }

close_audio_output_ports()
  {
     int i;

     for(i=0; i<100; i++)
       if(audio_output_ports[i] != 0)
	 {
	   ALcloseport(audio_output_ports[i]);
	   ALfreeconfig(conf[i]);
	 }
  }




/**********************************************************************
 *
 * Functions kept for OpenDVE_procs.c compatibility
 *
 *********************************************************************/
void QueryPluginStatus(DveConfParticipant *part)
{
}

void ProcessClipboardData(char *format, char *data, int length)
{
}



/********************************************************************
 *
 * Frames management
 *
 *******************************************************************/

/************************    GSM    *********************************/

void DecodeGSMFrame(gsm_frame *frames, int part)
{
  char buffer[5][320];
  int i;
  
  if(audio_output_ports[part] == 0)
    {
      OpenOutPort(part);
    }

 for(i=0; i<5; i++)
    {
      gsm_decode(handle, frames[i], buffer[i]);
    }
  
  ALwritesamps(audio_output_ports[part], buffer, 800);
}




void
SendGSMFrame()
{
   gsmData *msg;
   char sig[5][320];
   int size;
   int i;

   msg = (gsmData *) dveMallocDataPacket(dveSession,
                                          sizeof(gsmData));
   if (!msg)
   {
     printf("Message allocation error\n");
     return;
   }
 
   /* Get the audio from mic. */
   GetFrameFromMic(sig);
   for(i=0; i<5; i++)
     {
	gsm_encode(handle, sig[i], &(msg->frames[i]));
     }

   if(SendPluginMessage(dveSession,
                           AUDIO_PLUGIN_PORT, GSM, msg,
                           sizeof(gsmData)) < 0)
		printf("Sending failure\n");

}

/****************************   ADPCM  ********************************/

void DecodeADPCMFrame(char *inbuf, int part)
{
  char buffer[1600];
  
  /*  schedctl(NDPRI,NDPHIMIN); */

  if(audio_output_ports[part] == 0)
    {
      OpenOutPort(part);
    }
  
  adpcm_decoder(inbuf, buffer, 800, &adpcmstates[part]);
  
  ALwritesamps(audio_output_ports[part], buffer, 800);
}


void
SendADPCMFrame()
{
   audioPluginMessageStructure *msg;
   char sig[1600];
   int size;

   msg = (audioPluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(audioPluginMessageStructure));
   if (!msg)
   {
     printf("Message allocation error\n");
     return;
   }
 
   /* Get the audio from mic. */
   GetFrameFromMic(sig);
   adpcm_coder(sig, msg->buf, 800, &adpcmsendstate);

   if(SendPluginMessage(dveSession,
                           AUDIO_PLUGIN_PORT, ADPCM, msg,
                           sizeof(audioPluginMessageStructure)) < 0)
		printf("Sending failure\n");

}

/*******************************************************************
 *
 * Background process function for sending audio
 *
 *******************************************************************/
void SendFrame()
{
  schedctl(NDPRI,NDPNORMMAX); 

  InitMic();
  while(1)
    {
      if(current_state == START_STATE)
	{
	  switch(code_type)
	    {
	    case GSM:
	      SendGSMFrame();
	      break;
	      
	    case ADPCM:
	      SendADPCMFrame();
	      break;
	    }
	}
      else
	{
	  sleep(1);
	}
    }
  
  CloseMic();
}


/**************************************************************************
 *
 * Function processing incoming messages
 *
 *************************************************************************/
void 
ProcessPluginMessage(u_int32 messageType, void *data, int data_size, 
		     DveUserHandle sender) 
{
   switch (messageType){
      case ADPCM:
	 DecodeADPCMFrame(data, sender);
	 break;

      case GSM:
	 DecodeGSMFrame(data, sender);
	 break;
   }
}


/************************************************************************
 *
 * Conference mangement functions
 *
 ***********************************************************************/
void 
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

void 
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}



/*********************************************************************
 *
 * Plugin management functions
 *
 ********************************************************************/
void 
ShowPlugin(void)
{
   fprintf(stderr, "ShowPlugin\n");
}

void 
HidePlugin(void)
{
   fprintf(stderr, "HidePlugin\n");
}

void 
CleanupPlugin(void)
{
  CloseSpeaker();
}


/************************** END Plugin_procs.c **************************/






