/*******************************************************************
 *
 * Name:	TSAudioPluginMotif.c
 *
 * Description: Audio plugin for OpenDVE, GSM/ADPCM compression,
 *              multipoint
 *
 * Author:	Tomasz Stachowiak
 *
 * Date:	06/09/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#define _BSD_SIGNALS
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>

#include <Xm/Xm.h>	    /* Motif includes */
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/MainW.h>
#include <Xm/PanedW.h>

#include <dve/dve.h>	    /* DVE includes */
#include <dve/confMgr.h>

/*
 *    stuff for sproc(2) system call to fork a second thread.
 */
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/schedctl.h>

/*
 *    the following define enables 4.3BSD behavior for the kill(3B)
 *    system call, and the include file is necessary for this function.
 */
#define _BSD_SIGNALS
#include <signal.h>



#include "TSAudioPluginMotif.h"
#include "AudioPluginProtocol.h"

#define XT


extern DveSession dveSession;	/* global variables */
extern XtAppContext app;
extern int current_state;
extern XtWorkProcId workproc;
int code_type = GSM;

/* Sending audio process PID */
int send_pid = 0;

Widget start_but, stop_but;

void SendFrame();

void
QuitApplication(int signo)
{
   gsm_close();
   CleanupPlugin();
   exit(0);
}

/*
 * Main()
 */
void
main(int argc, char **argv)
{

  Widget toplevel, toprowcol, rowcol, scale, mic_scale,
	 main_w, menubar, option_menu;
  void start_pushed();
  void stop_pushed();
  void scale_change();
  void mic_scale_change();
  void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs);
  void option_cb();
  XmString label, str1, str2;


  signal(SIGCLD, SIG_IGN);

  InitAudio();
  init_audio_output_ports();
  initADPCMstates();
  VolumeChange(50);
  SetMicGain(127);

  current_state = STOP_STATE;

  gsm_init();

  send_pid = sproc(SendFrame, PR_SALL);
  if(send_pid == -1)
    {
      fprintf(stderr, "Unable to create sending thread...aborting\n");
      exit(-1);
    }

  /*
   * Motif interface initialization
   */

/* Application */
  toplevel = XtVaAppInitialize(&app, "Audio plugin", NULL, 0, 
			       &argc, argv, NULL,	   
			       NULL);
  
/* Main window */
  main_w =  XtVaCreateManagedWidget("main_w", xmMainWindowWidgetClass, 
				    toplevel,
				    NULL);
/* Menu bar */
  str1 = XmStringCreateSimple("File");
  menubar = XmVaCreateSimpleMenuBar(main_w, "menubar",
	    XmVaCASCADEBUTTON, str1, 'F',
	    NULL);
  XmStringFree(str1);
				    
/* Pulldown menu (File) */
  str1 = XmStringCreateSimple("Exit");
  str2 = XmStringCreateSimple("Ctrl+C");
  XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_menu,
	    XmVaPUSHBUTTON, str1, 'E', "Ctrl<Key>c", str2,
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);
  

/* Vertical rowcolum widget */
  toprowcol =  XtVaCreateManagedWidget("toprowcol", xmPanedWindowWidgetClass, 
				      main_w,
/*				      XmNorientation, XmVERTICAL,*/
				      NULL);

/* Horizontal rowcolumn widget (for buttons) */
  rowcol =  XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
				      toprowcol,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
/* Start button */  
  label = XmStringCreateSimple("START");
  start_but = XtVaCreateManagedWidget("start", xmPushButtonWidgetClass, rowcol,
				      XmNwidth, 100,
				      XmNalignment, XmALIGNMENT_CENTER,
				      XmNlabelString, label,
				      NULL);
  XmStringFree(label);

/* Stop button */
  label = XmStringCreateSimple("STOP");
  stop_but = XtVaCreateManagedWidget("stop", xmPushButtonWidgetClass, rowcol,
				     XmNlabelString, label,
				     NULL);
  XmStringFree(label);

  XtSetSensitive(stop_but, False);

/* Buttons callbacks setting */
  XtAddCallback(start_but, XmNactivateCallback, start_pushed, NULL);
  XtAddCallback(stop_but, XmNactivateCallback, stop_pushed, NULL);

  label = XmStringCreateSimple("Compression:");
  str1 = XmStringCreateSimple("GSM");
  str2 = XmStringCreateSimple("ADPCM");
  option_menu = XmVaCreateSimpleOptionMenu(toprowcol, "option_menu", 
		  label, 'C', 0, option_cb,
		  XmVaPUSHBUTTON, str1, 'G', NULL, NULL,
		  XmVaPUSHBUTTON, str2, 'A', NULL, NULL,
		  NULL);
  XmStringFree(str1);
  XmStringFree(str2);
  XmStringFree(label);

  
  XtManageChild(option_menu);
  XtManageChild(rowcol);

/* Volume scale */
  scale = XtVaCreateManagedWidget("scale", xmScaleWidgetClass, toprowcol,      
		XtVaTypedArg, XmNtitleString, XmRString, "Volume", 6,
		XmNorientation, XmHORIZONTAL,
	        XmNmaximum, 100,
		XmNminimum, 0,
		XmNvalue, 50,
		XmNshowValue, True,
		NULL);

/* Volume scale callback setting */
  XtAddCallback(scale, XmNvalueChangedCallback, scale_change, NULL);

/* Microphone scale */
  mic_scale = XtVaCreateManagedWidget("mic_scale", xmScaleWidgetClass, 
		toprowcol,      
		XtVaTypedArg, XmNtitleString, XmRString, "Microphone", 10,
		XmNorientation, XmHORIZONTAL,
	        XmNmaximum, 100,
		XmNminimum, 0,
		XmNvalue, 50,
		XmNshowValue, True,
		NULL);

/* Volume scale callback setting */
  XtAddCallback(mic_scale, XmNvalueChangedCallback, mic_scale_change, NULL);

  XtManageChild(menubar);
  XtRealizeWidget(toplevel);



   /*
    * Start up OpenDVE
    */
   if (InitializeOpenDVE(app, AUDIO_PLUGIN_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

  XtAppMainLoop(app);
}


/*
 * Buttons pushing callbacks
 */

void start_pushed()
{
  if (current_state == STOP_STATE)
    {
      current_state = START_STATE;
      XtSetSensitive(start_but, False);
      XtSetSensitive(stop_but, True);
      /*      workproc = XtAppAddWorkProc(app, SendFrame, NULL);*/
    }
}

void stop_pushed()
{
  if (current_state == START_STATE)
  {
    current_state = STOP_STATE;
    XtSetSensitive(stop_but, False);
    XtSetSensitive(start_but, True);
    /*    XtRemoveWorkProc(workproc);*/
  }
}

/*
 * Scale callback
 */
void
scale_change(w, client_data, cbs)
Widget w;
caddr_t client_data;
XmScaleCallbackStruct *cbs;
{
  VolumeChange(cbs->value);
}
/*
 * Scale callback
 */

void
mic_scale_change(w, client_data, cbs)
Widget w;
caddr_t client_data;
XmScaleCallbackStruct *cbs;
{
  SetMicGain(255 - (int)((float)cbs->value * (float)2.55));
}

/*
 * File menu callback
 */
void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
  if(item_no == 0)
    {
       close_audio_output_ports();
       if(send_pid != 0)
	 {
	   kill(send_pid, SIGKILL);
	 }
       exit(0);
    }
}

void
option_cb(w, item_no, cbs)
Widget w;
int item_no;
XmAnyCallbackStruct *cbs;
{
   switch(item_no)
     {
	case 0:
	code_type = GSM;
	break;

	case 1:
	code_type = ADPCM;
	break;
     }
}












